package com.persagy.netty.collect;

import org.springframework.stereotype.Component;

import com.persagy.constant.CommonConst;

import io.netty.bootstrap.Bootstrap;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelFutureListener;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelOption;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioSocketChannel;
import io.netty.handler.codec.string.StringDecoder;
import io.netty.handler.codec.string.StringEncoder;
import io.netty.util.CharsetUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * iot-collect 服务netty
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021年12月9日 上午10:09:00
 */
@Slf4j
@Component
public class IotCollectNettyClient {

    public static Channel channel;
    
    public Bootstrap bootstrap = new Bootstrap();
    
    public void connect() {
        EventLoopGroup workerGroup = new NioEventLoopGroup();
        try {
            bootstrap.group(workerGroup);
            bootstrap.channel(NioSocketChannel.class);
            bootstrap.option(ChannelOption.AUTO_READ, true);
            bootstrap.handler(new ChannelInitializer<SocketChannel>() { // 绑定连接初始化器
				@Override
				protected void initChannel(SocketChannel ch) throws Exception {
					log.info("正在连接中...");
					ch.pipeline().addLast(new StringEncoder(CharsetUtil.UTF_8));
					ch.pipeline().addLast(new StringDecoder(CharsetUtil.UTF_8));
				}
			});
            ChannelFuture f = bootstrap.connect(CommonConst.collectHost, CommonConst.collectPort).sync();
            channel = f.channel();
        } catch (Exception e) {
            log.info("连接异常", e);
        } 
    }


    public void reConnect() {
        // 加入断线后自动重连监听器
        channel = bootstrap.connect(CommonConst.collectHost, CommonConst.collectPort).addListener(new ChannelFutureListener() {
            @Override
            public void operationComplete(ChannelFuture future) throws Exception {
                if (future.cause() != null) {
                    log.error("Failed to connect: {}", future.cause());
                }
            }
        }).channel();
    }


    public void sendMessage(String command) throws InterruptedException {
    	int count = 0;
    	while ((!channel.isActive() || !channel.isWritable()) && count < 5) {
    		channel.close();
        	this.reConnect();
        	count++;
        	// 不能太快，稍微等一会在执行
        	Thread.yield();
    	}
    	if (count >= 5) {
    		log.error("Failed to connect iot-collect服务");
    		return;
		}
    	
    	log.info("上报数据至iot-collect服务，报文：{}", command);
        try {
            channel.writeAndFlush(command);
        } catch (Exception e) {
        	e.printStackTrace();
            log.error("上报数据至iot-collect服务失败", e);
            channel.close();
        }
    }

}
